(*


******* Fri  12-20-1991   11:42:06  ******


v2.00
   - added  NoAnswer

   - Detected Ringing. This was being added to the unknowns when in many cases
     there was a actual No Answer response or No Carrier situation

   - Added -H option to display all histograms

   - allowed for file path directory to be added.

   - Automatically creates a UNKNOWNS list showing when dials ended up with
     unrecognizable responses.

   - Automatically creates a HIGHCOST list showing when dials ended up with
     No Carrier responses.

*)


uses Tpcrt,dos,strings,tpstring,cmdline;

const
    version        : string[50] = 'V2.00';
    uline          : string     = '';
    line           : string     = '';
    Phone          : string     = '';
    dialline       : string     = ''; (* last number dialed *)
    nline          : longint    = 0;  (* line count in file *)
    total          : word       = 0;
    tries          : word       = 0;
    busy           : word       = 0;
    Connect        : word       = 0;
    NoCarrier      : word       = 0;
    NoDialTone     : word       = 0;
    NoAnswer       : word       = 0;
    unknown        : word       = 0;
    ByNode         : Boolean    = False;

Type HourAmountType = array[0..23] of word;

var
    dir           : searchrec;
    fv            : text;
    fvun          : text;
    fvno          : text;
    HGTries       : HourAmountType;
    HGBusy        : HourAmountType;
    HGConnect     : HourAmountType;
    HGNoCarrier   : HourAmountType;
    HGNoDialTone  : HourAmountType;
    HGUnKnown     : HourAmountType;
    HGNoAnswer    : HourAmountType;
    ConnectRate   : HourAmountType;
    i             : word;
    buf           : array[1..10240] of char;
    p             : byte;
    filepath      : pathstr;

Procedure DisplayAmount(HG : HourAmountType);
 var i : byte;
 begin
  for i := 0 to 23 do
    begin
       write('  Hour  ',i:2,':  ',HG[i]:4);
       if ((i+1) mod 2) = 0 then writeln;
    end;
 end;

Procedure CalculateConnectRate;
 var i : byte;
 begin
  for i := 0 to 23 do
    begin
       ConnectRate[i] := HGTries[i]
                         - HGBusy[i]
                         - HGNoDialTone[i]
                         - HGNoCarrier[i]
                         - HGNoAnswer[i]
                         - HGUnKnown[i];
    end;
 end;

Function MaxTimes(HG : HourAmountType): Word;
var i,max : word;
  begin
     max := 0;
     for i := 0 to 23 do if HG[i] > max then Max := HG[i];
     MaxTimes := Max;
  end;

Function MaxIndex(HG : HourAmountType): ShortInt;
var i,
    j,
    max : shortint;
  begin
     j   := -1;
     max := 0;
     for i := 0 to 23 do
        if HG[i] > max
           then begin
                 Max := Hg[i];
                 j  := i;
                end;
     MaxIndex := j;
  end;

Procedure DrawHistoGram(HG : HourAmountType);
 var i   : byte;
     max : word;
 begin
  max := MaxTimes(HG);
  if Max = 0 then max := 1;

  for i := 0 to 23 do
    begin
       writeln(i:2,':00 |', pad(repeatchr('',Trunc(65*HG[i]/Max)),65),HG[i]:5);
    end;
 end;

Procedure Increment;
var ok : boolean;
  begin
    inc(tries);
    ok := false;
    readln(fv,line);
    i := strint(copy(line,10,2));
    Inc(HGTries[i]);
    inc(nline);
    repeat
       if pos('Busy',line) > 0
          then begin
                 inc(busy);
                 Inc(HGBusy[i]);
                 ok := true;
               end
       else
       if pos('Connect',line) > 0
          then begin
                 inc(Connect);
                 inc(HGConnect[i]);
                 ok := true;
               end
       else
       if pos('No Carrier',line) > 0
          then begin
                 inc(NoCarrier);
                 inc(HGNoCarrier[i]);
                 writeln(fvno,' ',dialline);
                 ok := true;
               end
       else
       if pos('No Dial Tone',line) > 0
          then Begin
                inc(NoDialTone);
                inc(HGNoDialTone[i]);
                 ok := true;
               end
       else
       if pos('Ringing',line) > 0
          then Begin
                readln(fv,line);
                inc(nline);
               end
       else
       if pos('No Answer',line) > 0
          then begin
                 inc(NoAnswer);
                 inc(HGNoAnswer[i]);
                 ok := true;
               end
       else    Begin
                inc(unknown);
                inc(HGUnKnown[i]);
                writeln(fvun,' ',dialline);
                writeln(fvun,' ',line);
                ok := true;
               end
    until ok;
  end;

begin

     writeln('BUSYBBS ',version,' (c) copyright 1991 Santronics Software');
    if paramcount = 0 then
       begin
        writeln('BusyBBS - Counts the number of Call Attempts');
        writeln('usage> BusyBBS LogFile(s) -Pphone# -HGT -HGB -HGC -HGN');
        writeln;
        writeln('   -P    phone number or netail address, ie -P1:141:/880');
        writeln('   -HGT  Histogram on # of tries');
        writeln('   -HGB  Histogram on # of busy signals');
        writeln('   -HGC  Histogram on # of successful connects');
        writeln('   -HGN  Histogram on # of no carriers');
        writeln('   -HGA  Histogram on # of no answers');
        writeln('   -H    Display ALL Histograms');
        writeln;
        writeln('example> BusyBBS *.LOG -P804-723-1663 -HGT -HGB -HGC');
        writeln('example> BusyBBS *.LOG <<--- Report on All Numbers');
        writeln;
        writeln('NOTES:');
        writeln(' - For BUSYBBS to work best, Maximum VERBOSE LOGGING is required.');
        writeln(' - UNKNOWNS file is automatically created showing unknown dials');
        writeln(' - HIGHCOST file is automatically created showing NoCarrier dials');
        halt;
       end;

    fillchar(HGTries,sizeof(HGTries),0);
    fillchar(HGBusy,sizeof(HGBusy),0);
    fillchar(HGConnect,sizeof(HGConnect),0);
    fillchar(HGNoCarrier,sizeof(HGNoCarrier),0);
    fillchar(HGNoDialTone,sizeof(HGNoDialTone),0);
    fillchar(HGUnKnown,sizeof(HGUnKnown),0);
    fillchar(HGNoAnswer,sizeof(HGNoAnswer),0);

    if switch('-P',2,p) then Phone := copy(Paramstr(p),3,80);

    ByNode := pos(':',Phone) > 0;
    assign(output,''); rewrite(output);
    assign(fvun,'unknowns'); rewrite(fvun);
    assign(fvno,'highcost'); rewrite(fvno);

    filepath := justpathname(paramstr(1));
    findfirst(paramstr(1),anyfile,dir);
    while doserror = 0 do
      begin
        writeln(fvun,'FILE: ',addbackslash(filepath)+dir.name);
        writeln(fvno,'FILE: ',addbackslash(filepath)+dir.name);
        assign(fv,addbackslash(filepath)+dir.name);
        SettextBuf(fv,buf);
        reset(fv);
        nline := 0;
        fastwrite(pad('Reading '+addbackslash(filepath)+dir.name+'....',30),wherey,1,7);
        while not eof(fv) do
          begin
           readln(fv,line);
           inc(nline);
           if ByNode
              then begin
                    if pos('node '+Phone,line) > 0 then
                       begin
                          dialline := line;
                          readln(fv,line);
                          inc(nline);
                          Increment
                       end
                   end
              else Begin
                    if pos('Dialing '+Phone,line) > 0 then
                       begin
                        dialline := line;
                        Increment;
                       end;
                   end;
          end;
        close(fv);
        findnext(dir);
      end;

      close(fvun);
      close(fvno);

      total := Busy + Connect + NoCarrier + NoDialTone + Unknown + NoAnswer;
      if total = 0 then total := 1;
      if ByNode
         then writeln('Summary of Processing Node ',Phone)
         else writeln('Summary of Dialing ',Phone);
      writeln('Tries       : ',tries:5);
      writeln('  Busy      : ',busy:5, '    ', (100*( busy / Total)):5:2,'%');
      writeln('  Connect   : ',Connect:5, '    ', (100*( Connect / Total)):5:2,'%');
      writeln('  NoCarrier : ',NoCarrier:5, '    ', (100*( NoCarrier / Total)):5:2,'%');
      writeln('  NoDialTone: ',NoDialTone:5, '    ', (100*( NoDialtone / Total)):5:2,'%');
      writeln('  NoAnswer  : ',NoAnswer:5, '    ', (100*( NoAnswer / Total)):5:2,'%');
      writeln('  Unknown   : ',Unknown:5, '    ', (100*( Unknown / Total)):5:2,'%');
      total := Busy + Connect + NoCarrier + NoDialTone + Unknown + NoAnswer;
      Writeln('Total       : ',Total:5);
      writeln;
      Writeln('Busiest Hour: ',MaxIndex(HgBusy):5);
      Writeln('Best Hour   : ',MaxIndex(HGConnect):5);
      CalculateConnectRate;
      Writeln('Recommended Hour to Call : ', MaxIndex(ConnectRate));

      if switch('-HGT',0,p) or switch('-H',0,p) then
          begin
            Writeln('---- Tries  By the Hour');
            DrawHistoGram(HgTries);
          end;
      if switch('-HGB',0,p) or switch('-H',0,p) then
          begin
            Writeln('---- Busy   By the Hour');
            DrawHistoGram(HgBusy);
          end;
      if switch('-HGC',0,p) or switch('-H',0,p) then
          begin
            Writeln('---- Connect By the Hour');
            DrawHistoGram(HgConnect);
          end;
      if switch('-HGA',0,p) or switch('-H',0,p) then
          begin
            Writeln('---- NoAnswer By the Hour');
            DrawHistoGram(HgNoAnswer);
          end;
      if switch('-HGN',0,p) or switch('-H',0,p) then
          begin
            Writeln('---- NoCarriers By the Hour - High Cost ITEM!');
            DrawHistoGram(HgNoCarrier);
          end;

end.
